/* $Id: CA2G.cc,v 1.8 2002/09/04 15:56:11 mazina Exp $
 */


/* ========================================================================= */
/*                                                                           */
/*    Copyright 2003,                                                        */
/*      Rice University,                                                     */
/*      University of California-San Diego,                                  */
/*      University of California-Santa Barbara,                              */
/*      University of Chicago,                                               */
/*      University of Houston,                                               */
/*      University of Illinois,                                              */
/*      University of Southern California-Information Sciences Institute,    */
/*      University of Tennessee-Knoxville.                                   */
/*                                                                           */
/*    All Rights Reserved.                                                   */
/*                                                                           */
/* See GrADSoft_Authors file in the GrADSoft top-level directory             */
/*                           for a list of developers.                       */
/*                                                                           */ 
/* Permission to use, copy, modify and distribute any part of this source    */
/* file of the Grid Application Development Software (GrADS) project for     */
/* educational, research and non-profit purposes, without fee, and without a */
/* written agreement is hereby granted, provided that the above copyright    */
/* notice, this paragraph and the following two paragraphs appear in all     */
/* copies.                                                                   */
/*                                                                           */
/*                                                                           */
/* IN NO EVENT SHALL THE AUTHORS AND COPYRIGHT HOLDERS, EITHER JOINTLY OR    */
/* SEPARATELY, BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,         */
/* INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT */
/* OF THE USE OF THIS SOFTWARE, EVEN IF THEY HAVE BEEN ADVISED OF THE        */
/* POSSIBILITY OF SUCH DAMAGE.                                               */
/*                                                                           */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED HEREIN ON AN "AS IS" BASIS, AND THE AUTHORS AND */
/* COPYRIGHT HOLDERS HAVE NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,     */
/* UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE AUTHORS AND COPYRIGHT        */
/* HOLDERS MAKE NO REPRESENTATIONS AND EXTEND NO WARRANTIES OF ANY KIND,     */
/* EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED     */
/* WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, OR     */
/* THAT THE USE OF THIS SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR  */
/* OTHER RIGHTS.                                                             */
/*                                                                           */
/* ========================================================================= */


#include "CA2G.h"


void
usage() 
{
  cout << DNL << "usage: ";
  cout << "CA2G [-v] <file containing classad>" << DNL;
  cout << "where:" << DNL;
  cout << "   -v          turns on verbose progress output"         << NL;
  cout << NL;
  cout << endl;
  return;
}

int 
main(int argc, char **argv) 
{

  bool    verboseMessages = false;
  char*   inputFile;

  // need to dig out a more robust arg reader object,
  // probably from Rice DSystem code -- mmzn
  
  if ( (argc < 2) || (argc > 3) ) {
    usage();
    exit(1);
  }

  if ( string(argv[1]) == string("-v") ) {
    if ( argc != 3 ) {
      usage();
      exit(1);
    }
    verboseMessages = true;
    Syslog::setVerbosity(INFO);
    inputFile  =  argv[2];
  } else {
    inputFile  =  argv[1];
  } 

  ifstream caFILE(inputFile, ios::in);
  char in_buf[SIZE_INPUT_BUF];
  string* classad_text;

  if ( !caFILE ) {
    cout << NL << "ERROR: classAd input file is missing" << NL << endl;
    exit(1);
  }

  caFILE.getline(in_buf, SIZE_INPUT_BUF);
  if ( caFILE.eof() ) {
    cout << NL << "ERROR: classAd input file is empty" << NL << endl;
    exit(1);
  }

  if ( verboseMessages ) {
    cout << DNL << "CA2G translator:" << NL << endl;
    cout << "input classad:" << NL << endl;
  }

  classad_text = new string(in_buf);
  if ( verboseMessages ) cout << in_buf << endl;

  while ( ! caFILE.eof() ) {
    caFILE.getline(in_buf, SIZE_INPUT_BUF);
    if ( ! caFILE.eof() ) {
      *classad_text += in_buf;
      if ( verboseMessages ) cout << in_buf << endl;
    }
  }
  if ( verboseMessages ) cout << endl;

  // we don't support multiple scopes; replace the classAd scope
  // operator "." with "_" in the input if it follows "other"
  // to collapse the scope.
  string::size_type idx;
  if ( *classad_text == "" ) {
    cout << NL << "ERROR: classAd string is empty" << NL << endl;
    exit(1);
  }
  idx = 0;
  while ( idx != string::npos ) {
    idx = classad_text->find("other.");
    if ( idx != string::npos ) {
      classad_text->replace(idx, 6, "other_");
      if ( ( classad_text->substr(idx, 14) != "other_cpuspeed" )   &&
           ( classad_text->substr(idx, 14) != "other_LLatency" )   &&
           ( classad_text->substr(idx, 14) != "other_RLatency" )   &&
           ( classad_text->substr(idx, 16) != "other_LBandwidth" ) &&
           ( classad_text->substr(idx, 16) != "other_RBandwidth" ) &&
           ( classad_text->substr(idx, 24) != "other_physicalmemorysize" ) ) {
        cout << NL << "ERROR: unrecognized `other.' identifier" << NL << endl;
        exit(1);
      }
    }
  }

  ClassAdParser  parser;
  ClassAd        *input_classad;

  sexprExtract  s_expr_gen_rse;
  sexprExtract  s_expr_gen_resourcesOK;
  string        s_expr_for_rse;
  string        s_expr_for_resourcesOK;

  ofstream tFile("CA2GtransOut", ios::out);
  if ( !tFile ) {
    cout << NL << "ERROR: output file open fails" << NL << endl;
    exit(1);
  }

  tFile << "this file produced by CA2G translator." << endl;

  input_classad     = parser.ParseClassAd(*classad_text, true);

  // cout << "debug: parser return" << endl;

  if ( input_classad == (ClassAd *) NULL ) {
    cout << NL << "ERROR: parser: input classad invalid" << NL << endl;
    exit(1);
  }

  s_expr_gen_rse.SetDesiredEquationType(RANK);
  s_expr_gen_rse.Unparse( s_expr_for_rse, input_classad);
  if ( s_expr_for_rse.substr(0,6) == "ERROR:" ) {
    cout << NL << "On rank expression: " << NL;
    cout << s_expr_for_rse << NL << endl;
    exit(1);
  }
  if ( verboseMessages ) cout << "Rank s_expr: " << NL
	              << s_expr_for_rse << NL << endl;

  tFile << "\nrankFuncString = " << s_expr_for_rse << endl;

  s_expr_gen_resourcesOK.SetDesiredEquationType(REQMTS);
  s_expr_gen_resourcesOK.Unparse( s_expr_for_resourcesOK, input_classad);
  if ( s_expr_for_resourcesOK.substr(0,6) == "ERROR:" ) {
    cout << NL << "On requirements expression: " << NL;
    cout << s_expr_for_resourcesOK << NL << endl;
    exit(1);
  }
  if ( verboseMessages ) cout << "Reqmt s_expr: " << NL
	              << s_expr_for_resourcesOK << NL << endl;

  tFile << "\nreqmtsFuncString = " << s_expr_for_resourcesOK << endl;

  tFile << endl;
  tFile.close();

  exit(0);
}


