/* $Id: tCA_AART.cc,v 1.2 2002/08/31 00:29:33 mazina Exp $
 */


/* ========================================================================= */
/*                                                                           */
/*    Copyright 2003,                                                        */
/*      Rice University,                                                     */
/*      University of California-San Diego,                                  */
/*      University of California-Santa Barbara,                              */
/*      University of Chicago,                                               */
/*      University of Houston,                                               */
/*      University of Illinois,                                              */
/*      University of Southern California-Information Sciences Institute,    */
/*      University of Tennessee-Knoxville.                                   */
/*                                                                           */
/*    All Rights Reserved.                                                   */
/*                                                                           */
/* See GrADSoft_Authors file in the GrADSoft top-level directory             */
/*                           for a list of developers.                       */
/*                                                                           */ 
/* Permission to use, copy, modify and distribute any part of this source    */
/* file of the Grid Application Development Software (GrADS) project for     */
/* educational, research and non-profit purposes, without fee, and without a */
/* written agreement is hereby granted, provided that the above copyright    */
/* notice, this paragraph and the following two paragraphs appear in all     */
/* copies.                                                                   */
/*                                                                           */
/*                                                                           */
/* IN NO EVENT SHALL THE AUTHORS AND COPYRIGHT HOLDERS, EITHER JOINTLY OR    */
/* SEPARATELY, BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,         */
/* INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT */
/* OF THE USE OF THIS SOFTWARE, EVEN IF THEY HAVE BEEN ADVISED OF THE        */
/* POSSIBILITY OF SUCH DAMAGE.                                               */
/*                                                                           */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED HEREIN ON AN "AS IS" BASIS, AND THE AUTHORS AND */
/* COPYRIGHT HOLDERS HAVE NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,     */
/* UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE AUTHORS AND COPYRIGHT        */
/* HOLDERS MAKE NO REPRESENTATIONS AND EXTEND NO WARRANTIES OF ANY KIND,     */
/* EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED     */
/* WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, OR     */
/* THAT THE USE OF THIS SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR  */
/* OTHER RIGHTS.                                                             */
/*                                                                           */
/* ========================================================================= */

#include "syslog.h"
#include "aart.h"

// Translated ClassAd Translated (tCA) AART

#define SIZE_INPUT_BUF 4096   // these s_exprs can get big!

AART *
construct_aartModel_tCA(string *rankFuncStrPtr, string *reqmtsFuncStrPtr);


AART *
construct_aartModel() {
  string rankFuncString;
  string reqmtsFuncString;

  // file format, as written by CA2G:
  //
  //   this file produced by CA2G translator.
  //   <empty line>
  //   rankFuncString = (s-expr)
  //   <empty line>
  //   reqmtsFuncString = (s-expr)
  //   <empty line>
  //   <eof>
  //
  ifstream cf("CA2GtransOut", ios::in);
  char in_buf[SIZE_INPUT_BUF];

  if ( !cf ) {
    Syslog::log(ERROR, "CAT_AART", "CA2GtransOut file open fails");
    return( (AART *) NULL );
  }

  cf.getline(in_buf, SIZE_INPUT_BUF);
  if ( cf.eof() ) {
    Syslog::log(ERROR, "CAT_AART", "CA2GtransOut file is empty");
    return( (AART *) NULL );
  }

  cf.getline(in_buf, SIZE_INPUT_BUF);
  if ( cf.eof() ) {
    Syslog::log(ERROR, "CAT_AART", 
		       "CA2GtransOut file ends before rank found");
    return( (AART *) NULL );
  }

  cf.getline(in_buf, SIZE_INPUT_BUF);
  if ( cf.eof() ) {
    Syslog::log(ERROR, "CAT_AART", "CA2GtransOut file: no rank function");
    return( (AART *) NULL );
  }
  // skip "rankFuncString = " prefix;  given the prefix is 17 chars,
  // and a minimal valid s_expr is 1 char (e.g. literal n, 0 <= n < 10):
  if ( strlen(in_buf) < (size_t) 19 ) {
    Syslog::log(ERROR, "CAT_AART", 
		       "CA2GtransOut file: malformed rank function");
    return( (AART *) NULL );
  }
  rankFuncString = string(&in_buf[17]);

  cf.getline(in_buf, SIZE_INPUT_BUF);
  if ( cf.eof() ) {
    Syslog::log(ERROR, "CAT_AART", 
		       "CA2GtransOut file ends before reqmts found");
    return( (AART *) NULL );
  }

  cf.getline(in_buf, SIZE_INPUT_BUF);
  if ( cf.eof() ) {
    Syslog::log(ERROR, "CAT_AART", "CA2GtransOut file: no reqmts function");
    return( (AART *) NULL );
  }
  // skip "reqmtsFuncString = " prefix; 18 chars
  if ( strlen(in_buf) < (size_t) 20 ) {
    Syslog::log(ERROR, "CAT_AART", 
		       "CA2GtransOut file: malformed reqmts function");
    return( (AART *) NULL );
  }
  reqmtsFuncString = string(&in_buf[18]);

  bool printIt = false;
  if ( printIt ) {
    cout << endl << "rankFuncString = " << rankFuncString << endl;
    cout << endl << "reqmtsFuncString = " << reqmtsFuncString << endl;
  }

  return(construct_aartModel_tCA(&rankFuncString, &reqmtsFuncString));
}

AART *
construct_aartModel_tCA(string *rankFuncStrPtr, string *reqmtsFuncStrPtr) {

  FunctionalAttribute *CA_rank, *CA_reqmts;
  Constraint *cALL_1, *cALL_2;
  

  Syslog::setVerbosity(WARNING);

  // ----------------------------------------------------------
  // Rank 

  CA_rank = new FunctionalAttribute(ATOMIC, COMP, RANK,
				    "classad rank",
				    *rankFuncStrPtr,
	                            FLOATS_TO_FLOAT, RANK, NO_INPUT_ARGS);

  // ----------------------------------------------------------
  // Requirements

  CA_reqmts = new FunctionalAttribute(AGGREGATE, STORAGE, REQMTS,
				      "classad requirements",
				      *reqmtsFuncStrPtr,
	                              FLOATS_TO_BOOL, REQMTS, NO_INPUT_ARGS);


  cALL_1 = new Constraint(false,1.0,CA_rank);
  cALL_2 = new Constraint(false,1.0,CA_reqmts);

  // Create a 1-D Mesh AART
  AART *aartCCC;
  aartCCC = new AART(MESH, 1, "translated_classAd");

  // Add application-wide constraints - complain if not added
  if ( ! aartCCC->AddApplicationConstraint(cALL_1) ) 
      cout << "AddApplicationConstraint cALL_1 fails\n";
  if ( ! aartCCC->AddApplicationConstraint(cALL_2) ) 
      cout << "AddApplicationConstraint cALL_2 fails\n";

  return(aartCCC);
}

