/** $Id: tCA_ASCFC.cc,v 1.5 2003/03/11 20:10:52 mazina Exp $
 */


/* ========================================================================= */
/*                                                                           */
/*    Copyright 2003,                                                        */
/*      Rice University,                                                     */
/*      University of California-San Diego,                                  */
/*      University of California-Santa Barbara,                              */
/*      University of Chicago,                                               */
/*      University of Houston,                                               */
/*      University of Illinois,                                              */
/*      University of Southern California-Information Sciences Institute,    */
/*      University of Tennessee-Knoxville.                                   */
/*                                                                           */
/*    All Rights Reserved.                                                   */
/*                                                                           */
/* See GrADSoft_Authors file in the GrADSoft top-level directory             */
/*                           for a list of developers.                       */
/*                                                                           */ 
/* Permission to use, copy, modify and distribute any part of this source    */
/* file of the Grid Application Development Software (GrADS) project for     */
/* educational, research and non-profit purposes, without fee, and without a */
/* written agreement is hereby granted, provided that the above copyright    */
/* notice, this paragraph and the following two paragraphs appear in all     */
/* copies.                                                                   */
/*                                                                           */
/*                                                                           */
/* IN NO EVENT SHALL THE AUTHORS AND COPYRIGHT HOLDERS, EITHER JOINTLY OR    */
/* SEPARATELY, BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,         */
/* INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT */
/* OF THE USE OF THIS SOFTWARE, EVEN IF THEY HAVE BEEN ADVISED OF THE        */
/* POSSIBILITY OF SUCH DAMAGE.                                               */
/*                                                                           */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED HEREIN ON AN "AS IS" BASIS, AND THE AUTHORS AND */
/* COPYRIGHT HOLDERS HAVE NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,     */
/* UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE AUTHORS AND COPYRIGHT        */
/* HOLDERS MAKE NO REPRESENTATIONS AND EXTEND NO WARRANTIES OF ANY KIND,     */
/* EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED     */
/* WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, OR     */
/* THAT THE USE OF THIS SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR  */
/* OTHER RIGHTS.                                                             */
/*                                                                           */
/* ========================================================================= */

#include "syslog.h"
#include "tCA_ASCFC.h"
#include "gridCfgInfo.h"

///////////////////////////////////////////////////////////////////////
// ASCFC general methods

// std constructor
catASCFC::catASCFC(AttListPtr U_ConfigListPtr, 
                   AttListPtr P_ConfigListPtr,
                   RunPcsVectorPtr RunPiecesPtr) 
                 : ASCFC(U_ConfigListPtr, P_ConfigListPtr, RunPiecesPtr) {

  Syslog::setVerbosity(WARNING);

  Syslog::log(DEBUG, "catASCFC", "constructor called");

  UserConfigListPtr = U_ConfigListPtr;
  ProbConfigListPtr = P_ConfigListPtr;
  ProbRunPiecesPtr = RunPiecesPtr;

  ASCFnameSuffix = ".cfg";
}

catASCFC::~catASCFC() { } 

bool
catASCFC::build_ASCF() {

  Attribute *a;
  char in_buf[SIZE_INPUT_BUF];

  a = searchAttList(UserConfigListPtr, 
                    DISCRETE, CONFIG, USER, MYGRIDFILE);
  if ( a == (Attribute *) NULL ) {
    Syslog::log(ERROR, "catASCFC", "no CONFIG.USER.MYGRIDFILE found");
    return(false);
  } 
  string gridCfgFile = ((DiscreteAttribute *) a)->getValue();
  
  gridCfgInfo *gridCI = new gridCfgInfo(gridCfgFile);

  // assume local_executable_name & local_appConfig_file same for all subjobs 

  string ExecFileName = (*ProbRunPiecesPtr)[0].local_executable_name;
  string Cfg_FileName = ExecFileName + ASCFnameSuffix;
  string cm_config_file = (*ProbRunPiecesPtr)[0].local_cmConfig_file;

  for (unsigned int i=0; i < (*ProbRunPiecesPtr).size(); i++) {
    (*ProbRunPiecesPtr)[i].local_appConfig_file = Cfg_FileName;
    (*ProbRunPiecesPtr)[i].local_working_directory =
		gridCI->lookUp_lwd((*ProbRunPiecesPtr)[i].resource_name);
    (*ProbRunPiecesPtr)[i].clusterID = 
		gridCI->lookUp_Cid((*ProbRunPiecesPtr)[i].resource_name);
    (*ProbRunPiecesPtr)[i].site_NFS_ID =
                gridCI->lookUp_NFSid((*ProbRunPiecesPtr)[i].resource_name);
    (*ProbRunPiecesPtr)[i].site_gatekeeper_port =
                gridCI->lookUp_gate_port((*ProbRunPiecesPtr)[i].resource_name);
    (*ProbRunPiecesPtr)[i].site_gsiftp_port =    
              gridCI->lookUp_gsiftp_port((*ProbRunPiecesPtr)[i].resource_name);
    (*ProbRunPiecesPtr)[i].site_LD_LIBRARY_PATH =    
              gridCI->lookUp_LD_LIBRARY_PATH((*ProbRunPiecesPtr)[i].resource_name);
    (*ProbRunPiecesPtr)[i].site_gradsIS_cmd =    
              gridCI->lookUp_gradsIS_cmd((*ProbRunPiecesPtr)[i].resource_name);
    if ( (*ProbRunPiecesPtr)[i].local_working_directory 
                                                == string("NOT FOUND") ) {
      Syslog::log(ERROR, "catASCFC", "lookUp_lwd() failure");
      return(false);
    } 
    if ( (*ProbRunPiecesPtr)[i].clusterID == string("UNKNOWN CLUSTER") ) {
      Syslog::log(ERROR, "catASCFC", "lookUp_Cid() failure");
      return(false);
    } 
    // failure of lookUp_NFSid() is not critical;
    // just means we do duplicate work in file staging
    // failure of lookUp_*_port() will just leave us with the std port.
  }

  a = searchAttList(UserConfigListPtr, 
                    DISCRETE, CONFIG, USER, AUTOPILOT_MGR_MACHINE);
  if ( a == (Attribute *) NULL ) {
    Syslog::log(ERROR, "catASCFC", 
		       "no CONFIG.USER.AUTOPILOT_MGR_MACHINE found");
    return(false);
  } 
  string apMgr = ((DiscreteAttribute *) a)->getValue();

  a = searchAttList(UserConfigListPtr, DISCRETE, CONFIG, USER, CM_METRICS);
  if ( a == (Attribute *) NULL ) {
    Syslog::log(ERROR, "catASCFC", "no CONFIG.USER.CM_METRICS found");
    return(false);
  }
  string ap_metric_file = ((DiscreteAttribute *) a)->getValue();
  
  a = searchAttList(ProbConfigListPtr, DISCRETE, CONFIG, PROBLEM, APP_NAME);
  if ( a == (Attribute *) NULL ) {
    Syslog::log(ERROR, "catASCFC", "no CONFIG.PROBLEM.APP_NAME found");
    return(false);
  } 
  string autopilot_ident = ((DiscreteAttribute *) a)->getValue();
 
  a = searchAttList(ProbConfigListPtr, 
		    DISCRETE, CONFIG, PROBLEM, APPRESULTSFILE);
    if ( a == (Attribute *) NULL ) {
        Syslog::log(ERROR, "catASCFC", 
			   "no CONFIG.PROBLEM.APPRESULTSFILE found");
	return(false);
    }  
  string result_file = ((DiscreteAttribute *) a)->getValue();

  ofstream cfgFile(Cfg_FileName.c_str(), ios::out);
  if ( !cfgFile ) {
    Syslog::log(ERROR, "catASCFC", "output file open fails");
    return(false);
  }

  a = searchAttList(ProbConfigListPtr, 
                    DISCRETE, CONFIG, PROBLEM, APPCFGFILE_LOCATION);
  // if we don't find this, we assume user has no problem specific
  // config info to add to the ASCF and just skip this section
  if ( a != (Attribute *) NULL ) {
    string appCfgFile = ((DiscreteAttribute *) a)->getValue();
    ifstream acf(appCfgFile.c_str(), ios::in);
    if ( !acf ) {
      Syslog::log(ERROR, "catASCFC", "APPCFGFILE_LOCATION file open fails");
      return(false);
    }
    acf.getline(in_buf, SIZE_INPUT_BUF);
    // an empty APPCFGFILE_LOCATION file is not considered an error
    while ( ! acf.eof() ) {
      cfgFile << in_buf << endl;
      acf.getline(in_buf, SIZE_INPUT_BUF);
    }
  } 

  bool UsingSwapMgr = false;
  string ActiveResourcePool;  
  string ActiveResourcePct;  
  string sm_machine;
  string sm_port;
  a = searchAttList(UserConfigListPtr, 
                    DISCRETE, CONFIG, USER, ACTIVE_RESOURCE_PCT);
  // if we don't find this, we assume user isn't using the swap
  // manager.  If we do find it, we assume the appMgr has already
  // checked that the remaining needed attributes are there.
  if ( a != (Attribute *) NULL ) {
    UsingSwapMgr = true;
    ActiveResourcePct = ((DiscreteAttribute *) a)->getValue();
    a = searchAttList(UserConfigListPtr, 
                    DISCRETE, CONFIG, USER, SWAP_MGR_MACHINE);
    if ( a == (Attribute *) NULL ) {
      Syslog::log(ERROR, "catASCFC", "missing SWAP_MGR_MACHINE");
      return(false);
    }
    sm_machine = ((DiscreteAttribute *) a)->getValue();
    a = searchAttList(UserConfigListPtr, 
                    DISCRETE, CONFIG, USER, SWAP_MGR_PORT);
    if ( a == (Attribute *) NULL ) {
      Syslog::log(ERROR, "catASCFC", "missing SWAP_MGR_PORT");
      return(false);
    }
    sm_port = ((DiscreteAttribute *) a)->getValue();
    // 
    // if user didn't specify an ACTIVE_RESOURCE_POOL, the appMgr was
    // supposed to add this attribute after the scheduler decided
    // on the value...
    a = searchAttList(UserConfigListPtr, 
                    DISCRETE, CONFIG, USER, ACTIVE_RESOURCE_POOL);
    if ( a == (Attribute *) NULL ) {
      Syslog::log(ERROR, "catASCFC", "missing ACTIVE_RESOURCE_POOL");
      return(false);
    }
    ActiveResourcePool = ((DiscreteAttribute *) a)->getValue();
  } 

 
  // standard cactus can't deal with these, it tried to parse them
  // like cactus parameters and aborts when it can't
  //
  // GrADS modified cactus will stop parsing the .par file upon
  // finding the following line
  //
  //     EndCactusInput =
  //
  // "=" char is necessary for Cactus ParseFile routine
  //
  if ( UsingSwapMgr ) {
    // --double piffle--, the swap manager code doesn't use the same syntax!!?
    //    cfgFile << "Swap_Mgr: " << sm_machine        << endl;  
    //    cfgFile << "SM_port: "  << sm_port           << endl;
    //    cfgFile << "Active_Resources: "   << ActiveResourcePool << endl;
    cfgFile << endl << "[mpi_swap]" << endl << endl;
    cfgFile << "swap manager =  " << sm_machine << ":"  << sm_port << endl;  
    cfgFile << "ap = "   << ActiveResourcePool << endl;
    cfgFile << "id = " << autopilot_ident << endl;
    cfgFile << "log = /tmp/mpi_swap_" << autopilot_ident << ".log" << endl;
    cfgFile << endl << "[monitoring]" << endl << endl;
  }
  cfgFile << "results: "    << result_file     << endl;  
  cfgFile << "AP_Mgr: "     << apMgr           << endl;  
  cfgFile << "AP_ident: "   << autopilot_ident << endl;
  cfgFile << "AP_metrics: " << ap_metric_file  << endl;
  cfgFile << "CM_config: "  << cm_config_file  << endl;
  cfgFile.close();

  return(true);
}


ASCFC *CreateASCFCObject(AttListPtr U_ConfigListPtr,
                         AttListPtr P_ConfigListPtr,
                         RunPcsVectorPtr RunPiecesPtr)
{
  catASCFC *catASCFCptr = new catASCFC(U_ConfigListPtr, 
                                       P_ConfigListPtr, 
                                       RunPiecesPtr);
  return((ASCFC *)catASCFCptr);
}

