/* $Id: tCA_RSE.cc,v 1.2 2002/09/12 13:42:15 mazina Exp $
 */


/* ========================================================================= */
/*                                                                           */
/*    Copyright 2003,                                                        */
/*      Rice University,                                                     */
/*      University of California-San Diego,                                  */
/*      University of California-Santa Barbara,                              */
/*      University of Chicago,                                               */
/*      University of Houston,                                               */
/*      University of Illinois,                                              */
/*      University of Southern California-Information Sciences Institute,    */
/*      University of Tennessee-Knoxville.                                   */
/*                                                                           */
/*    All Rights Reserved.                                                   */
/*                                                                           */
/* See GrADSoft_Authors file in the GrADSoft top-level directory             */
/*                           for a list of developers.                       */
/*                                                                           */ 
/* Permission to use, copy, modify and distribute any part of this source    */
/* file of the Grid Application Development Software (GrADS) project for     */
/* educational, research and non-profit purposes, without fee, and without a */
/* written agreement is hereby granted, provided that the above copyright    */
/* notice, this paragraph and the following two paragraphs appear in all     */
/* copies.                                                                   */
/*                                                                           */
/*                                                                           */
/* IN NO EVENT SHALL THE AUTHORS AND COPYRIGHT HOLDERS, EITHER JOINTLY OR    */
/* SEPARATELY, BE LIABLE TO ANY PARTY FOR DIRECT, INDIRECT, SPECIAL,         */
/* INCIDENTAL, OR CONSEQUENTIAL DAMAGES, INCLUDING LOST PROFITS, ARISING OUT */
/* OF THE USE OF THIS SOFTWARE, EVEN IF THEY HAVE BEEN ADVISED OF THE        */
/* POSSIBILITY OF SUCH DAMAGE.                                               */
/*                                                                           */
/*                                                                           */
/* THIS SOFTWARE IS PROVIDED HEREIN ON AN "AS IS" BASIS, AND THE AUTHORS AND */
/* COPYRIGHT HOLDERS HAVE NO OBLIGATION TO PROVIDE MAINTENANCE, SUPPORT,     */
/* UPDATES, ENHANCEMENTS, OR MODIFICATIONS. THE AUTHORS AND COPYRIGHT        */
/* HOLDERS MAKE NO REPRESENTATIONS AND EXTEND NO WARRANTIES OF ANY KIND,     */
/* EITHER IMPLIED OR EXPRESS, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED     */
/* WARRANTIES OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE, OR     */
/* THAT THE USE OF THIS SOFTWARE WILL NOT INFRINGE ANY PATENT, TRADEMARK OR  */
/* OTHER RIGHTS.                                                             */
/*                                                                           */
/* ========================================================================= */

#include <stdio.h>

#include "syslog.h"
#include "tCA_RSE.h"

///////////////////////////////////////////////////////////////////////
// translated classAd RSE general methods


/// std constructor
catRSE::catRSE(RSSM* inputRSSM, AttListPtr ConfigListPtr) 
	                        : RSE(inputRSSM, ConfigListPtr) {

  Syslog::setVerbosity(WARNING);

  Syslog::log(DEBUG, "catRSE", "constructor called");
 
  copRSSM = inputRSSM;
  ProbConfigListPtr = ConfigListPtr;

}


catRSE::~catRSE() {

  Syslog::log(DEBUG, "catRSE", "destructor called");

}

int
catRSE::GetExecutionTimeCost(Schedule* sched,
			     ResourceBag* gridInfo,
			     Network* network,
			     double* execCost)
{
  EnvContextPtr dvp = new ResourceDataVector();
  if ( ! dvp->LoadData(sched, gridInfo, network) ) {
    Syslog::log(ERROR, "catRSE", "ResourceDataVector.LoadData() fails.");
    return(1);
  }
  Syslog::log(DEBUG, "catRSE", "finished resource LoadData().");

  catRSSM *rssmPtr = (catRSSM *) copRSSM;
  bool evalSuccess;
  double CA_rank_value;

  rssmPtr->Rank_Function->setEnvContextPtr(dvp);
  rssmPtr->Rank_Function->setEnvFuncPtr(&GetResourceProperty);
  CA_rank_value = rssmPtr->Rank_Function->evalFloatFunction(&evalSuccess);
  if ( ! evalSuccess ) {
    Syslog::log(ERROR, "catRSE", "CA Rank_Function fails");
    return(1);
  }

  Syslog::log(DEBUG, "catRSE", "finished eval Rank.");
  *execCost = CA_rank_value;

  return(0);
}

int
catRSE::GetExecutionTimeCost(EnvContextPtr dvp, double* execCost)
{
  catRSSM *rssmPtr = (catRSSM *) copRSSM;
  bool evalSuccess;
  double CA_rank_value;

  if ( dvp == (EnvContextPtr) NULL ) {
    Syslog::log(ERROR, "catRSE", "CA Rank_Function; EnvContextPtr is NULL");
    return(1);
  }

  rssmPtr->Rank_Function->setEnvContextPtr(dvp);
  rssmPtr->Rank_Function->setEnvFuncPtr(&GetResourceProperty);
  CA_rank_value = rssmPtr->Rank_Function->evalFloatFunction(&evalSuccess);
  if ( ! evalSuccess ) {
    Syslog::log(ERROR, "catRSE", "CA Rank_Function fails");
    return(1);
  }

  Syslog::log(DEBUG, "catRSE", "finished eval Rank.");
  *execCost = CA_rank_value;

  return(0);
}


RSE *CreateRSEObject(RSSM* inputRSSM, AttListPtr ConfigListPtr)
{
  catRSE *catRSEptr = new catRSE(inputRSSM, ConfigListPtr);
  return((RSE *)catRSEptr);
}

